package zero1hd.rhythmbullet.audio;

import java.io.IOException;
import java.util.Map;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

import org.tritonus.share.sampled.TAudioFormat;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Music.OnCompletionListener;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.math.MathUtils;

import zero1hd.rhythmbullet.audio.wavedecoder.AudioSampleReader;


public class Mp3Manager implements MusicManager {
	private int readWindowSize = 1024;
	private Music music;
	private int playbackIndex, readIndex;
	private FileHandle fileHandle;
	AudioInputStream in;
	private AudioInputStream ais;
	private AudioFormat af;
	private AudioSampleReader d;
	private Map<String, Object> properties;
	
	public Mp3Manager(FileHandle file) {
		this.fileHandle = file;
		try {
			in = AudioSystem.getAudioInputStream(file.file());
		    AudioFormat baseFormat = in.getFormat();
		    properties = ((TAudioFormat)baseFormat).properties();
			af = new AudioFormat(
					AudioFormat.Encoding.PCM_SIGNED,
					baseFormat.getSampleRate(),
					16, baseFormat.getChannels(), 
					baseFormat.getChannels()*2,
					baseFormat.getSampleRate(), false);
			ais = AudioSystem.getAudioInputStream(baseFormat, in);
			d = new AudioSampleReader(ais);
			music = Gdx.audio.newMusic(file);
		} catch (UnsupportedAudioFileException | IOException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void dispose() {
		music.dispose();
		try {
			ais.close();
			in.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void playbackIndexUpdate() {
		playbackIndex = (int) ((getDuration()*getSampleRate())/getReadWindowSize());
	}

	@Override
	public int getPlaybackIndexPosition() {
		return playbackIndex;
	}

	@Override
	public int getReadWindowSize() {
		return readWindowSize;
	}

	@Override
	public int readSamples(float[] samples) {
		try {
			readIndex++;
			return d.readSamples(samples);
		} catch (IOException e) {
			e.printStackTrace();
		};
		return 0;
	}

	@Override
	public long getSampleCount() {
		return ais.getFrameLength();
	}

	@Override
	public float getDuration() {
		return ((long) properties.get("duration"))/1000000f;
	}

	@Override
	public float getSampleRate() {
		return af.getSampleRate();
	}

	@Override
	public void pause() {
		music.pause();
	}

	@Override
	public void play() {
		music.play();
	}

	@Override
	public boolean isPlaying() {
		return music.isPlaying();
	}

	@Override
	public float getPositionInSeconds() {
		
		return music.getPosition();
	}

	@Override
	public void setPosition(float position) {
		music.setPosition(position);
	}

	@Override
	public void setOnCompletionListener(OnCompletionListener listener) {
		music.setOnCompletionListener(listener);
	}

	@Override
	public void setVolume(float percent) {
		music.setVolume(percent);
	}

	@Override
	public boolean isFinishedLoading() {
		return true;
	}

	@Override
	public String getBasicSongName() {
		return fileHandle.nameWithoutExtension();
	}

	@Override
	public FileHandle getMusicFile() {
		return fileHandle;
	}

	public void seek(long position) {
		closeStream();
		reInitStream();
		skip(position);
	}
	
	private void closeStream() {
		if (ais != null) {
			try {
				ais.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	private void reInitStream() {
		try {
			AudioInputStream in = AudioSystem.getAudioInputStream(fileHandle.file());
			ais = AudioSystem.getAudioInputStream(af, in);
			in.close();
		} catch (UnsupportedAudioFileException | IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void synchronize() {
		seek(MathUtils.round(((Integer) properties.get("audio.length.bytes")).intValue() * (getPositionInSeconds()/getDuration())));
	}

	@Override
	public int getReadIndex() {
		return readIndex;
	}

	public void skip(long bytes) {
		long totalSkipped = 0;
		long skipped = 0;
		
		while (totalSkipped < bytes) {
			try {
				skipped = ais.skip(bytes-totalSkipped);
			} catch (IOException e) {
				e.printStackTrace();
			}
			
			if (skipped == 0) break;
			totalSkipped += skipped;
		}
	}

	@Override
	public void skipReadWindow() {
		skip(readWindowSize*af.getFrameSize());
		readIndex++;
	}

}
